<?php

namespace Drupal\commerce_shepa\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Url;

/**
 * Provides the Off-site Redirect payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "commerce_shepa_redirect",
 *   label = " shepa.com (Off-site redirect)",
 *   display_label = "shepa.com",
 *    forms = {
 *     "offsite-payment" =
 *   "Drupal\commerce_shepa\PluginForm\OffsiteRedirect\PaymentOffsiteForm",
 *   },
 *   payment_method_types = {"credit_card"},
 * )
 */
class OffsiteRedirect extends OffsitePaymentGatewayBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
        'api_key' => 'sandbox',
      ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['api_key'] = [
      '#type' => 'textfield',
      '#title' => t('API Key'),
      '#default_value' => $this->configuration['api_key'],
      '#description' => t("The API Key which is provided by shepa.com. If you use the gateway in the Test mode, just enter the 'test' word."),
      '#required' => TRUE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      // Save configuration
      $this->configuration['api_key'] = $values['api_key'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {

    // The parameters which is posted to the callback url by shepa.com
    $status = $_GET['status'];
    $token = $_GET['token'];
    $api_key = $this->configuration['api_key'];
    preg_match_all('!\d+!', $order->getTotalPrice(), $amounts);
    $amount = 0;
    if(!empty($amounts)) $amount = $amounts[0][0];
    // Prevents double spending:
    // If a bad manner user have a successful transaction and want
    // to have another payment with previous transId, we must prevent them.
    $query = \Drupal::entityQuery('commerce_payment')
      ->condition('remote_state', $token);
    $payments = $query->execute();
    if (count($payments) > 0) {
      \Drupal::logger('commerce_shepa')
        ->error('commerce_shepa: Double spending occured on order <a href="@url">%order</a> from ip @ip', [
          '@url' => Url::fromUri('base:/admin/commerce/orders/' . $order->id())
            ->toString(),
          '%order' => $order->id(),
          '@ip' => $order->getIpAddress(),
        ]);
      drupal_set_message('Double spending occured.', 'error');
      /** @var \Drupal\commerce_checkout\Entity\CheckoutFlowInterface $checkout_flow */
      $checkout_flow = $order->checkout_flow->entity;
      $checkout_flow_plugin = $checkout_flow->getPlugin();
      $redirect_step = $checkout_flow_plugin->getPreviousStepId('payment');
      $checkout_flow_plugin->redirectToStep($redirect_step);
    }


    $verify_url = 'https://merchant.shepa.com/api/v1/verify';

    if ($this->configuration["mode"] == "test") {
      $api_key = 'sandbox';
      $verify_url = 'https://sandbox.shepa.com/api/v1/verify';
    }
    // Status of the transaction which is posted to the callback url by shepa.com
    if ($status == "success") {
      try {
          $params = [
              "api" => $api_key,
              "amount" => $amount,
              "token" => $token,
          ];
          $ch = curl_init();
          curl_setopt($ch, CURLOPT_URL, $verify_url);
          curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));
          curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
          curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
          curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
          curl_setopt($ch, CURLOPT_HTTPHEADER, [
              'Content-Type: application/json',
          ]);
          $res = curl_exec($ch);
          curl_close($ch);

        // Verification result
        $result = json_decode($res);
        // Status of the transaction verification
        if (!empty($result->success)) {
          $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
          $payment = $payment_storage->create([
            'state' => 'completed',
            'amount' => $order->getTotalPrice(),
            'payment_gateway' =>  $this->parentEntity->id(),
            'order_id' => $order->id(),
            'remote_id' => $result->result->refid,
            'authorized' => $this->time->getRequestTime(),
          ]);
          $payment->save();

          drupal_set_message($this->t('Payment was processed.'));
        }
        else {
          drupal_set_message(
            ' shepa.com: status: failed'
            . ' & errorCode:<br> ' . implode("<br>",$result->error) , 'error');
        }
      } catch (\Exception $e) {
        drupal_set_message('Error: ' . $e->getMessage(), 'error');
      }

    }
    else {
      throw new PaymentGatewayException("Failed");
    }
  }
}
